<?php

/*
 * Copyright (c) 2012, Steve "Sc00bz" Thomas <steve AT tobtu DOT com>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the organization nor the names of its contributors
 *       may be used to endorse or promote products derived from this software
 *       without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

require_once 'EncryptBcrypt.php';

// ******************************
// **** Start of tinfoil hat ****
// ******************************

// Use 160 bit encryption or higher since quantum computers exist and they square root the key space (Grover's algorithm).
// With classical computers 80 bit encryption or higher is considered safe ((2 ^ 80) ^ 2 = 2 ^ 160).

// ****************************
// **** End of tinfoil hat ****
// ****************************


// **********************************
// **** Start of AES256-CBC test ****
// **********************************

echo "<h1>Starting AES256-CBC test</h1>\n";
$NIST_AES256_CBC_TEST_VECTORS = array(
	'ddc6bf790c15760d8d9aeb6f9a75fd4e', '0a6bdc6d4c1e6280301fd8e97ddbe601', '9b80eefb7ebe2d2b16247aa0efc72f5d', '7f2c5ece07a98d8bee13c51177395ff7',
	'7818d800dcf6f4be1e0e94f403d1e4c2', 'e74cd1c92f0919c35a0324123d6177d3', '8092a4dcf2da7e77e93bdd371dfed82e', '49af6b372135acef10132e548f217b17',
	'8bcd40f94ebb63b9f7909676e667f1e7', 'fe1cffb83f45dcfb38b29be438dbd3ab', '0dc58a8d886623705aec15cb1e70dc0e', 'c218faa16056bd0774c3e8d79c35a5e4',
	'047bba83f7aa841731504e012208fc9e', 'dc8f0e4915fd81ba70a331310882f6da', '1569859ea6b7206c30bf4fd0cbfac33c', '300ade92f88f48fa2df730ec16ef44cd',
	'1fe6cc3c05965dc08eb0590c95ac71d0', '59e858eaaa97fec38111275b6cf5abc0', '2239455e7afe3b0616100288cc5a723b', '3ee500c5c8d63479717163e55c5c4522',
	'd5e38bf15f16d90e3e214041d774daa8', 'b1f4066e6f4f187dfe5f2ad1b17819d0', '6ef4cc4de49b11065d7af2909854794a', 'ac86bc606b6640c309e782f232bf367f',
	'36aff0ef7bf3280772cf4cac80a0d2b2', '1f8eedea0f62a1406d58cfc3ecea72cf', 'abf4154a3375a1d3e6b1d454438f95a6', '96f96e9d607f6615fc192061ee648b07',
	'cf37cdaaa0d2d536c71857634c792064', 'fbd6640c80245c2b805373f130703127', '8d6a8afe55a6e481badae0d146f436db', '6a4981f2915e3e68af6c22385dd06756',
	'42a1136e5f8d8d21d3101998642d573b', '9b471596dc69ae1586cee6158b0b0181', '753665c4af1eff33aa8b628bf8741cfd', '9a682acf40be01f5b2a4193c9a82404d',
	'54fafe26e4287f17d1935f87eb9ade01', '49d541b2e74cfe73e6a8e8225f7bd449', '11a45530f624ff6f76a1b3826626ff7b', 'f96b0c4a8bc6c86130289f60b43b8fba',
	'48c7d0e80834ebdc35b6735f76b46c8b', '2463531ab54d66955e73edc4cb8eaa45', 'ac9bd8e2530469134b9d5b065d4f565b', '3f5f9106d0e52f973d4890e6f37e8a00',
	'20ebc86f1304d272e2e207e59db639f0', 'e67ae6426bf9526c972cff072b52252c', '1a518dddaf9efa0d002cc58d107edfc8', 'ead731af4d3a2fe3b34bed047942a49f',
	'b1d4efe40242f83e93b6c8d7efb5eae9', 'cd2b1fec11fd906c5c7630099443610a', 'a1853fe47fe29289d153161d06387d21', '4632154179a555c17ea604d0889fab14',
	'dd27cac6401a022e8f38f9f93e774417', 'c090313eb98674f35f3123385fb95d4d', 'cc3526262b92f02edce548f716b9f45c', 'c0838d1a2b16a7c7f0dfcc433c399c33',
	'0d9ac756eb297695eed4d382eb126d26', '56ede9dda3f6f141bff1757fa689c3e1', '768f520efe0f23e61d3ec8ad9ce91774', 'b1144ddfa75755213390e7c596660490',
	'1d7c0c4040b355b9d107a99325e3b050', 'd8e2bb1ae8ee3dcf5bf7d6c38da82a1a', 'faf82d178af25a9886a47e7f789b98d7', '9b58dbfd77fe5aca9cfc190cd1b82d19',
	'77f392089042e478ac16c0c86a0b5db5', '19f08e3420ee69b477ca1420281c4782', 'a1b19beee4e117139f74b3c53fdcb875', 'a37a5869b218a9f3a0868d19aea0ad6a',
	'bc3594e865bcd0261b13202731f33580', '811441ce1d309eee7185e8c752c07557', '959971ce4134190563518e700b9874d1', '76b5614a042707c98e2132e2e805fe63',
	'7d9fa6a57530d0f036fec31c230b0cc6', '964153a83bf6989a4ba80daa91c3e081', 'a013014d4ce8054cf2591d06f6f2f176', 'd1c5f6399bf382502e385eee1474a869',
	'0007e20b8298ec354f0f5fe7470f36bd', 'b95ba05b332da61ef63a2b31fcad9879', '4620a49bd967491561669ab25dce45f4', '12e71214ae8e04f0bb63d7425c6f14d5',
	'4cc42fc1407b008fe350907c092e80ac', '08b244ce7cbc8ee97fbba808cb146fda', '39b333e8694f21546ad1edd9d87ed95b', '3b271f8ab2e6e4a20ba8090f43ba78f3',
	'9ad983f3bf651cd0393f0a73cccdea50', '8f476cbff75c1f725ce18e4bbcd19b32', '905b6267f1d6ab5320835a133f096f2a', '145b60d6d0193c23f4221848a892d61a',
	'55cfb3fb6d75cad0445bbc8dafa25b0f', '7b8e7098e357ef71237d46d8b075b0f5', '2bf27229901eb40f2df9d8398d1505ae', '83a63402a77f9ad5c1e931a931ecd706',
	'6f8ba6521152d31f2bada1843e26b973', 'e5c3b8e30fd2d8e6239b17b44bd23bbd', '1ac1f7102c59933e8b2ddc3f14e94baa', '21d9ba49f276b45f11af8fc71a088e3d',
	'649f1cddc3792b4638635a392bc9bade', 'e2775e4b59c1bc2e31a2078c11b5a08c', '2be1fae5048a25582a679ca10905eb80', 'da86f292c6f41ea34fb2068df75ecc29',
	'220df19f85d69b1b562fa69a3c5beca5', '1f11d5d0355e0b556ccdb6c7f5083b4d', '62526b78be79cb384633c91f83b4151b', '90ddbcb950843592dd47bbef00fdc876',
	'2fd0e41c5b8402277354a7391d2618e2', '3cdf13e72dee4c581bafec70b85f9660', 'afa2ffc137577092e2b654fa199d2c43', '8d683ee63e60d208e343ce48dbc44cac',
	'705a4ef8ba2133729c20185c3d3a4763', '0861a861c3db4e94194211b77ed761b9', '4b00c27e8b26da7eab9d3a88dec8b031', '5f397bf03084820cc8810d52e5b666e9',
	'63fafabb72c07bfbd3ddc9b1203104b8', '683e2140585b18452dd4ffbb93c95df9', '286894e48e537f8763b56707d7d155c8', 'a423deabc173dcf7e2c4c53e77d37cd1',
	'eb8168313e1cfdfdb5e986d5429cf172', '27127daafc9accd2fb334ec3eba52323', 'ee0715b96f72e3f7a22a5064fc592f4c', '29ee526770f2a11dcfa989d1ce88830f',
	'0493370e054b09871130fe49af730a5a', '9b7b940f6c509f9e44a4ee140448ee46', '2915be4a1ecfdcbe3e023811a12bb6c7', '7240e524bc51d8c4d440b1be55d1062c',
	'da63039d38cb4612b2dc36ba26684b93', '0f59cb5a4b522e2ac56c1a64f558ad9a', '7bfe9d876c6d63c1d035da8fe21c409d', 'acdace8078a32b1a182bfa4987ca1347');

$key = pack('H*', '0000000000000000000000000000000000000000000000000000000000000000');
$iv  = pack('H*', '00000000000000000000000000000000');
for ($a = 0; $a < 128; $a++)
{
	$bin = str_repeat('1', $a + 1) . str_repeat('0', 128 - ($a + 1));
	$pt = pack('H*',
		base_convert(substr($bin,  0, 32), 2, 16) . base_convert(substr($bin, 32, 32), 2, 16) .
		base_convert(substr($bin, 64, 32), 2, 16) . base_convert(substr($bin, 96, 32), 2, 16));
	$ctWithPadding = mcrypt_cbc(MCRYPT_RIJNDAEL_128, $key, $pt, MCRYPT_ENCRYPT, $iv); // AES 256
	$ct = bin2hex(substr($ctWithPadding, 0, 16));
	if ($NIST_AES256_CBC_TEST_VECTORS[$a] != $ct)
	{
		die("AES256-CBC is broken.");
	}
}
echo "Passed<br>\n";

// ********************************
// **** End of AES256-CBC test ****
// ********************************


// ******************************
// **** Start of bcrypt test ****
// ******************************

echo "<h1>Starting bcrypt test</h1>\n";
$hash0 = crypt(pack('H*', '2020ff'), '$2y$05$0000000000000000000000');
$hash1 = crypt(pack('H*', 'ffffff'), '$2y$05$0000000000000000000000');
if ($hash0 == $hash1 || !EncryptBcrypt::validBcryptHash($hash0))
{
	die("bcrypt is broken.");
}
echo "Passed<br>\n";

// ****************************
// **** End of bcrypt test ****
// ****************************


// **************************************
// **** Start of encrypted hash test ****
// **************************************

echo "<h1>Starting encrypted hash test</h1>\n";

// **** Create hash ****
echo "Creating hash...<br>\n";
$pw = 'lamepassword';
$start = microtime(true);
$hash = EncryptBcrypt::createEncryptedHash($pw);
$end = microtime(true);
if (EncryptBcrypt::validEncryptedHash($hash))
{
	echo "$hash *** Valid hash ***<br>\n";
}
else
{
	echo "$hash *** Invalid hash ***<br>\n";
}
echo 'Took ' . round(1000 * ($end - $start), 2) . ' ms (' . round(1 / ($end - $start), 2) . " new hashes/sec)<br>\n<br>\n";

// **** Test hash ****
echo "Testing hash...<br>\n";
$start = microtime(true);
$ans = EncryptBcrypt::testEncryptedHash($pw, $hash);
$end = microtime(true);
if ($ans)
{
	echo "Correct<br>\n";
}
else
{
	echo "Wrong<br>\n";
}
echo 'Took ' . round(1000 * ($end - $start), 2) . ' ms (' . round(1 / ($end - $start), 2) . " tests/sec)<br>\n<br>\n";

// **** Test wrong password ****
echo "Testing wrong password...<br>\n";
$start = microtime(true);
$ans = EncryptBcrypt::testEncryptedHash($pw . '1', $hash);
$end = microtime(true);
if ($ans)
{
	echo "Correct<br>\n";
}
else
{
	echo "Wrong<br>\n";
}
echo 'Took ' . round(1000 * ($end - $start), 2) . ' ms (' . round(1 / ($end - $start), 2) . " tests/sec)<br>\n<br>\n";

// **** Create 72 character password hash ****
echo "Creating 72 character password hash...<br>\n";
$pw = '123456789012345678901234567890123456789012345678901234567890123456789012';
$start = microtime(true);
$hash = EncryptBcrypt::createEncryptedHash($pw);
$end = microtime(true);
if (EncryptBcrypt::validEncryptedHash($hash))
{
	echo "$hash *** Valid hash ***<br>\n";
}
else
{
	echo "$hash *** Invalid hash ***<br>\n";
}
echo 'Took ' . round(1000 * ($end - $start), 2) . ' ms (' . round(1 / ($end - $start), 2) . " new hashes/sec)<br>\n<br>\n";

// **** Test with 73 character password hash ****
echo "Testing with 73 character password hash...<br>\n";
$start = microtime(true);
$ans = EncryptBcrypt::testEncryptedHash($pw . '3', $hash);
$end = microtime(true);
if ($ans)
{
	echo "Correct<br>\n";
}
else
{
	echo "Wrong<br>\n";
}
echo 'Took ' . round(1000 * ($end - $start), 2) . ' ms (' . round(1 / ($end - $start), 2) . " tests/sec)<br>\n<br>\n";

// **** Generating test data ****
echo "Generating 100 test rows...<br>\n";
$dbHost     = 'localhost';
$dbUserName = 'user';
$dbPassword = 'password';
$dbName     = 'test_db';
$mysqli = new mysqli($dbHost, $dbUserName, $dbPassword, $dbName);
if ($mysqli->connect_errno)
{
	die('Connect failed: ' . $mysqliIn->connect_error);
}
if (!$mysqli->query("CREATE TABLE IF NOT EXISTS `test_table` (`id` int(10) unsigned NOT NULL AUTO_INCREMENT, `hash` varchar(255) CHARACTER SET ascii COLLATE ascii_bin NOT NULL, PRIMARY KEY (`id`)) ENGINE=MyISAM DEFAULT CHARSET=latin1"))
{
	die("MySQL error: " . $mysqli->error);
}
if (!$mysqli->query("TRUNCATE TABLE `test_table`"))
{
	die("MySQL error: " . $mysqli->error);
}
$start = microtime(true);
for ($i = 0; $i < 100; $i++)
{
	$hash = EncryptBcrypt::createEncryptedHash('lamepassword');
	if ($hash === false)
	{
		die("Error creating hash.");
	}
	$hash = $mysqli->real_escape_string($hash);
	if (!$mysqli->query("INSERT INTO `test_table` SET `hash`='$hash'"))
	{
		die("MySQL error: " . $mysqli->error);
	}
}
$end = microtime(true);
$mysqli->close();
echo 'Took ' . round(1000 * ($end - $start), 2) . " ms<br>\n<br>\n";

// **** Changing encryption key ****
echo "Changing encryption key...<br>\n";
$start = microtime(true);
EncryptBcrypt::changeDatabaseEncryptionKey($dbHost, $dbUserName, $dbPassword, $dbName, 'id', 'hash', 'test_table');
$end = microtime(true);
echo 'Took ' . round(1000 * ($end - $start), 2) . " ms<br>\n<br>\n";
unlink('set_server_key-old.php');

// **** Test encryption key ****
echo "Testing encryption key...<br>\n";
$start = microtime(true);
$ret = EncryptBcrypt::testDatabaseEncryptionKey($dbHost, $dbUserName, $dbPassword, $dbName, 'id', 'hash', 'test_table');
$end = microtime(true);
if ($ret == -1)
{
	echo "Failed<br>\n";
}
elseif ($ret == 0)
{
	echo "All hashes are valid<br>\n";
}
else
{
	echo $ret . " hashes are invalid<br>\n";
}
echo 'Took ' . round(1000 * ($end - $start), 2) . " ms<br>\n<br>\n";

// ************************************
// **** End of encrypted hash test ****
// ************************************

?>